local RegisteringHandler = FlexBar2.OO.Mixin 
{
	"RegisterEvent", -- Custom RegisterEvent function, Registers the event with AceEvent if it ain't already. It is table-based, read its code for details ;)
	"UnregisterEvent",
	"RegisterScript", -- Custom version of the SetScript function. Very similar to my custom RegisterEvent function except it is for SetScript stuff :)
	"UnregisterScript",
	"UnregisterAll",
};

function RegisteringHandler:RegisterEvent(Event, MethodName)
	FlexBar2:Debug("Registering " , Event, " with " , MethodName);
	if(type(self[MethodName]) ~= "function") then
		return false
	end
	if(not FlexBar2:IsEventRegistered(Event)) then
		FlexBar2[Event] = function(...)
			for n ,u in pairs(FlexBar2.EventTable[Event]) do
				for _,v in pairs(u) do
					n[v](n, Event, ...);
				end
			end
		end
		FlexBar2:RegisterEvent(Event);
	end
	if(type(FlexBar2.EventTable[Event]) ~= "table") then
		FlexBar2.EventTable[Event] = {};
	end
	if(type(FlexBar2.EventTable[Event][self]) ~= "table") then
		FlexBar2.EventTable[Event][self] = {};
	end
	table.insert(FlexBar2.EventTable[Event][self], MethodName);
end

function RegisteringHandler:UnregisterEvent(Event, MethodName)
	if(MethodName) then
		return FlexBar2:RemoveByValue(FlexBar2.EventTable[Event][self], MethodName);
	else
		for n,v in pairs(FlexBar2.EventTable[Event][self]) do
			table.remove(FlexBar2.EventTable[Event][self], n);
		end
	end
end


function RegisteringHandler:RegisterScript(Trigger, MethodName)
	if(type(self.ScriptTable) ~= "table") then
		self.ScriptTable = {};
	end
	if(type(self.Frame:GetScript(Trigger)) ~= "function") then
		self.Frame:SetScript(Trigger, function(...)
			for n, v in ipairs(self.ScriptTable[Trigger]) do
				self[v](self, Trigger, ...);
				if(Trigger ~= "OnUpdate") then FlexBar2:Debug("Triggering ", v, " For Handler: ", Trigger); end
			end
		end);
	end
	if(type(self.ScriptTable[Trigger]) ~= "table") then
		self.ScriptTable[Trigger] = {};
	end
	table.insert(self.ScriptTable[Trigger], MethodName);
end

function RegisteringHandler:UnregisterScript(Trigger, MethodName)
	if(type(self.ScriptTable) ~= "table") then
		self.ScriptTable = {};
	end
	if(MethodName) then
		return FlexBar2:RemoveByValue(self.ScriptTable[Trigger], MethodName);
	else
		for n,v in pairs(self.ScriptTable[Trigger]) do
			table.remove(self.ScriptTable[Trigger], n);
		end
	end
end


FlexBar2:RegisterButtonClass(RegisteringHandler);
